@@ -11,7 +11,7 @@ use Term::ANSIColor ();
 use Sys::Hostname qw( hostname );
 
 # ABSTRACT: Parallel SSH client
-our $VERSION = '0.99_01'; # TRIAL VERSION
+our $VERSION = '0.99_02'; # TRIAL VERSION
 
 
 sub main
@@ -20,6 +20,8 @@ sub main
   $clad->run;
 }
 
+# this hook is used for testing
+# see t/args.t subtest 'color'
 our $_stdout_is_terminal = sub { -t STDOUT };
 
 sub new
@@ -64,12 +66,11 @@ sub new
   $self->{clusters} = [ split ',', shift @ARGV ];
   $self->{command}  = [ @ARGV ];
 
-  if(my $expanded = $self->aliases->{$self->command->[0]})
+  if(my $expanded = $self->alias->{$self->command->[0]})
   {
     if(ref $expanded)
     {
-      shift @{ $self->command };
-      unshift @{ $self->command }, @$expanded;
+      splice @{ $self->command }, 0, 1, @$expanded;
     }
     else
     {
@@ -103,13 +104,24 @@ sub user           { shift->{user}        }
 sub server         { shift->{server}      }
 sub verbose        { shift->{verbose}     }
 sub serial         { shift->{serial}      }
-sub aliases        { shift->config->aliases( default => {} ) }
 sub server_command { shift->config->server_command( default => 'clad --server' ) }
 sub ssh_command    { shift->config->ssh_command(    default => 'ssh' ) }
 sub ssh_options    { shift->config->ssh_options(    default => [ -o => 'StrictHostKeyChecking=no', 
                                                                  -o => 'BatchMode=yes',
                                                                  -o => 'PasswordAuthentication=no',
                                                                  '-T', ] ) }
+sub ssh_extra      { shift->config->ssh_extra(      default => [] ) }
+
+sub alias
+{
+  my($self) = @_;
+  $self->config->alias( default => sub {
+    my %deprecated = $self->config->aliases( default => {} );
+    say STDERR "use of aliases key in configuration is deprecated, use alias instead"
+        if %deprecated;
+    \%deprecated;
+  });
+}
 
 sub ret
 {
@@ -199,9 +211,66 @@ sub run_server
 {
   my($self) = @_;
   
-  my $input = Load(do { local $/; <STDIN> });
+  # Payload:
+  #
+  #   command: required, must be a array with at least one element
+  #     the command to execute
+  #
+  #   env: optional, must be a hash reference
+  #     any environmental overrides
+  #
+  #   verbose: optional true/false
+  #     print out extra diagnostics
+  #
+  #   version: required number or 'dev'
+  #     the client version
+  #
+  #   require: optional, number or 'dev'
+  #     specifies the minimum required server
+  #     server should die if requirement isn't met
+  #     ignored if set to 'dev'
+  #
+  
+  my $raw = do { local $/; <STDIN> };
+  my $input = eval { Load($raw) };
+  
+  if(my $yaml_error = $@)
+  {
+    say STDERR "Clad Server: side YAML Error:";
+    say STDERR $yaml_error;
+    say STDERR "payload:";
+    say STDERR $raw;
+    return 2;
+  }
   
   print STDERR Dump($input) if $input->{verbose};
+
+  if(ref $input->{command} ne 'ARRAY' || @{ $input->{command} } == 0)
+  {
+    say STDERR "Clad Server: Unable to find command";
+    return 2;
+  }
+  
+  if(defined $input->{env} && ref $input->{env} ne 'HASH')
+  {
+    say STDERR "Clad Server: env is not hash";
+    return 2;
+  }
+  
+  unless($input->{version})
+  {
+    say STDERR "Clad Server: no client version";
+    return 2;
+  }
+  
+  if($input->{require} && defined $App::clad::VERSION)
+  {
+    if($input->{require} ne 'dev' && $input->{require} > $App::clad::VERSION)
+    {
+      say STDERR "Clad Server: client requested version @{[ $input->{require} ]} but this is only $App::clad::VERSION";
+      return 2;
+    }
+  }
   
   $ENV{$_} = $input->{env}->{$_} for keys %{ $input->{env} };
   
@@ -209,12 +278,12 @@ sub run_server
   
   if($? == -1)
   {
-    say STDERR "failed to execute on @{[ hostname ]}";
+    say STDERR "Clad Server: failed to execute on @{[ hostname ]}";
     return 2;
   }
   elsif($? & 127)
   {
-    say STDERR "died with signal @{[ $? & 127 ]} on @{[ hostname ]}";
+    say STDERR "Clad Server: died with signal @{[ $? & 127 ]} on @{[ hostname ]}";
     return 2;
   }
   
@@ -229,6 +298,8 @@ use AE;
 use AnyEvent::Open3::Simple 0.76;
 use YAML::XS qw( Dump );
 
+our $VERSION = '0.99_02'; # TRIAL VERSION
+
 sub new
 {
   my($class, %args) = @_;
@@ -277,11 +348,13 @@ sub new
     env     => $args{env},
     command => $clad->command,
     verbose => $clad->verbose,
+    version => $App::clad::VERSION // 'dev',
   });
   
   $ipc->run(
     $clad->ssh_command,
     $clad->ssh_options,
+    $clad->ssh_extra,
     ($args{user} ? ('-l' => $args{user}) : ()),
     $args{host},
     $clad->server_command,
@@ -333,7 +406,7 @@ App::clad - Parallel SSH client
 
 =head1 VERSION
 
-version 0.99_01
+version 0.99_02
 
 =head1 SYNOPSIS
 
